@@ -1,8 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,7 +24,7 @@ Command line interface
 
 from bleachbit.Cleaner import backends, create_simple_cleaner, register_cleaners
 from bleachbit import _, APP_VERSION
-from bleachbit import Diagnostic, Options, Worker
+from bleachbit import SystemInformation, Options, Worker
 
 import logging
 import optparse
@@ -63,7 +63,7 @@ def cleaners_list():
     list(register_cleaners())
     for key in sorted(backends):
         c_id = backends[key].get_id()
-        for (o_id, o_name) in backends[key].get_options():
+        for (o_id, _o_name) in backends[key].get_options():
             yield "%s.%s" % (c_id, o_id)
 
 
@@ -81,17 +81,33 @@ def preview_or_clean(operations, really_clean):
         pass
 
 
-def args_to_operations(args, preset):
-    """Read arguments and return list of operations"""
+def args_to_operations_list(preset, all_but_warning):
+    """For --preset and --all-but-warning return list of operations as list
+
+    Example return: ['google_chrome.cache', 'system.tmp']
+    """
+    args = []
+    if not backends:
+        list(register_cleaners())
+    assert(len(backends) > 1)
+    for key in sorted(backends):
+        c_id = backends[key].get_id()
+        for (o_id, _o_name) in backends[key].get_options():
+            # restore presets from the GUI
+            if preset and Options.options.get_tree(c_id, o_id):
+                args.append('.'.join([c_id, o_id]))
+            elif all_but_warning and not backends[c_id].get_warning(o_id):
+                args.append('.'.join([c_id, o_id]))
+    return args
+
+
+def args_to_operations(args, preset, all_but_warning):
+    """Read arguments and return list of operations as dictionary"""
     list(register_cleaners())
     operations = {}
-    if preset:
-        # restore presets from the GUI
-        for key in sorted(backends):
-            c_id = backends[key].get_id()
-            for (o_id, o_name) in backends[key].get_options():
-                if Options.options.get_tree(c_id, o_id):
-                    args.append('.'.join([c_id, o_id]))
+    if not args:
+        args = []
+    args = set(args + args_to_operations_list(preset, all_but_warning))
     for arg in args:
         if 2 != len(arg.split('.')):
             logger.warning(_("not a valid cleaner: %s"), arg)
@@ -101,9 +117,10 @@ def args_to_operations(args, preset):
         if '*' == option_id:
             if cleaner_id in operations:
                 del operations[cleaner_id]
-            operations[cleaner_id] = []
-            for (option_id2, o_name) in backends[cleaner_id].get_options():
-                operations[cleaner_id].append(option_id2)
+            operations[cleaner_id] = [
+                option_id2
+                for (option_id2, _o_name) in backends[cleaner_id].get_options()
+            ]
             continue
         # add the specified option
         if cleaner_id not in operations:
@@ -142,6 +159,7 @@ def process_cmd_line():
                       help=_("launch the graphical interface"))
     parser.add_option('--exit', action='store_true',
                       help=optparse.SUPPRESS_HELP)
+
     if 'nt' == os.name:
         uac_help = _("do not prompt for administrator privileges")
     else:
@@ -153,6 +171,8 @@ def process_cmd_line():
                       help=optparse.SUPPRESS_HELP)
     parser.add_option("--preset", action="store_true",
                       help=_("use options set in the graphical interface"))
+    parser.add_option("--all-but-warning", action="store_true",
+                      help=_("enable all options that do not have a warning"))
     if 'nt' == os.name:
         parser.add_option("--update-winapp2", action="store_true",
                           help=_("update winapp2.ini, if a new version is available"))
@@ -162,6 +182,13 @@ def process_cmd_line():
                       help=_("output version information and exit"))
     parser.add_option('-o', '--overwrite', action='store_true',
                       help=_('overwrite files to hide contents'))
+
+    def expand_context_menu_option(option, opt, value, parser):
+        setattr(parser.values, 'gui', True)
+        setattr(parser.values, 'exit', True)
+
+    parser.add_option("--context-menu", action="callback", callback=expand_context_menu_option)
+
     (options, args) = parser.parse_args()
 
     cmd_list = (options.list_cleaners, options.wipe_free_space,
@@ -178,11 +205,11 @@ def process_cmd_line():
         pass
     if options.debug_log:
         logger.addHandler(logging.FileHandler(options.debug_log))
-        logger.info(Diagnostic.diagnostic_info())
+        logger.info(SystemInformation.get_system_information())
     if options.version:
         print("""
 BleachBit version %s
-Copyright (C) 2008-2020 Andrew Ziem.  All rights reserved.
+Copyright (C) 2008-2021 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
@@ -215,11 +242,12 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         for wipe_path in args:
             logger.info('Wiping free space in path: %s', wipe_path)
             import bleachbit.FileUtilities
-            for ret in bleachbit.FileUtilities.wipe_path(wipe_path):
+            for _ret in bleachbit.FileUtilities.wipe_path(wipe_path):
                 pass
         sys.exit(0)
     if options.preview or options.clean:
-        operations = args_to_operations(args, options.preset)
+        operations = args_to_operations(
+            args, options.preset, options.all_but_warning)
         if not operations:
             logger.error(_("No work to do. Specify options."))
             sys.exit(1)
@@ -247,7 +275,7 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         preview_or_clean(operations, True)
         sys.exit(0)
     if options.sysinfo:
-        print(Diagnostic.diagnostic_info())
+        print(SystemInformation.get_system_information())
         sys.exit(0)
     if not did_something:
         parser.print_help()
