@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -90,11 +90,10 @@ class Cleaner:
         for (option_id, __name) in self.get_options():
             try:
                 for cmd in self.get_commands(option_id):
-                    for dummy in cmd.execute(False):
-                        return False
-                for ds in self.get_deep_scan(option_id):
-                    if isinstance(ds, dict):
+                    for _dummy in cmd.execute(False):
                         return False
+                for _ds in self.get_deep_scan(option_id):
+                    return False
             except Exception:
                 logger = logging.getLogger(__name__)
                 logger.exception('exception in auto_hide(), cleaner=%s, option=%s',
@@ -105,8 +104,7 @@ class Cleaner:
         """Get list of Command instances for option 'option_id'"""
         for action in self.actions:
             if option_id == action[0]:
-                for cmd in action[1].get_commands():
-                    yield cmd
+                yield from action[1].get_commands()
         if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
@@ -115,8 +113,7 @@ class Cleaner:
         for action in self.actions:
             if option_id == action[0]:
                 try:
-                    for ds in action[1].get_deep_scan():
-                        yield ds
+                    yield from action[1].get_deep_scan()
                 except StopIteration:
                     return
         if option_id not in self.options:
@@ -475,13 +472,17 @@ class System(Cleaner):
 
             def gtk_purge_items():
                 """Purge GTK items"""
-                Gtk.RecentManager().purge_items()
+                Gtk.RecentManager().get_default().purge_items()
                 yield 0
 
-            for pathname in ["~/.recently-used.xbel", "~/.local/share/recently-used.xbel"]:
-                pathname = os.path.expanduser(pathname)
-                if os.path.lexists(pathname):
-                    yield Command.Shred(pathname)
+            xbel_pathnames = [
+                    '~/.recently-used.xbel',
+                    '~/.local/share/recently-used.xbel*',
+                    '~/snap/*/*/.local/share/recently-used.xbel']
+            for path1 in xbel_pathnames:
+                for path2 in glob.iglob(os.path.expanduser(path1)):
+                    if os.path.lexists(path2):
+                        yield Command.Shred(path2)
             if HAVE_GTK:
                 # Use the Function to skip when in preview mode
                 yield Command.Function(None, gtk_purge_items, _('Recent documents list'))
@@ -505,18 +506,7 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirname1 = os.path.expandvars(
-                "$USERPROFILE\\Local Settings\\Temp\\")
-            dirname2 = os.path.expandvars(r'%temp%')
-            dirname3 = os.path.expandvars("%windir%\\temp\\")
-            dirnames = []
-            if Windows.get_windows_version() >= 6.0:
-                # Windows Vista or later
-                dirnames.append(dirname2)
-            else:
-                # Windows XP
-                dirnames.append(dirname1)
-            dirnames.append(dirname3)
+            dirnames = [os.path.expandvars(r'%temp%'), os.path.expandvars("%windir%\\temp\\")]
             # whitelist the folder %TEMP%\Low but not its contents
             # https://bugs.launchpad.net/bleachbit/+bug/1421726
             for dirname in dirnames:
@@ -552,7 +542,7 @@ class System(Cleaner):
         if HAVE_GTK and 'clipboard' == option_id:
             def clear_clipboard():
                 clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-                clipboard.set_text(' ',1)
+                clipboard.set_text(' ', 1)
                 clipboard.clear()
                 return 0
             yield Command.Function(None, clear_clipboard, _('Clipboard'))
@@ -566,10 +556,9 @@ class System(Cleaner):
                 display = _("Overwrite free disk space %s") % pathname
 
                 def wipe_path_func():
-                    for ret in FileUtilities.wipe_path(pathname, idle=True):
-                        # Yield control to GTK idle because this process
-                        # is very slow.  Also display progress.
-                        yield ret
+                    # Yield control to GTK idle because this process
+                    # is very slow.  Also display progress.
+                    yield from FileUtilities.wipe_path(pathname, idle=True)
                     yield 0
                 yield Command.Function(None, wipe_path_func, display)
 
@@ -593,11 +582,6 @@ class System(Cleaner):
             for path in Windows.get_recycle_bin():
                 recycled_any = True
                 yield Command.Delete(path)
-            # If there were any files deleted, Windows XP will show the
-            # wrong icon for the recycle bin indicating it is not empty.
-            # The icon will be incorrect until logging in to Windows again
-            # or until it is emptied using the Windows API call for emptying
-            # the recycle bin.
 
             # Windows 10 refreshes the recycle bin icon when the user
             # opens the recycle bin folder.
@@ -686,15 +670,13 @@ def register_cleaners(cb_progress=lambda x: None, cb_done=lambda: None):
     # register CleanerML cleaners
     from bleachbit import CleanerML
     cb_progress(_('Loading native cleaners.'))
-    for ret in CleanerML.load_cleaners(cb_progress):
-        yield ret
+    yield from CleanerML.load_cleaners(cb_progress)
 
     # register Winapp2.ini cleaners
     if 'nt' == os.name:
         cb_progress(_('Importing cleaners from Winapp2.ini.'))
         from bleachbit import Winapp
-        for ret in Winapp.load_cleaners(cb_progress):
-            yield ret
+        yield from Winapp.load_cleaners(cb_progress)
 
     cb_done()
 
@@ -722,9 +704,7 @@ def create_simple_cleaner(paths):
                 if os.path.isdir(path):
                     for child in children_in_directory(path, True):
                         yield Command.Shred(child)
-                    yield Command.Shred(path)
-                else:
-                    yield Command.Shred(path)
+                yield Command.Shred(path)
     provider = CustomFileAction(None)
     cleaner.add_action('files', provider)
     return cleaner
@@ -741,8 +721,7 @@ def create_wipe_cleaner(path):
     display = _("Overwrite free disk space %s") % path
 
     def wipe_path_func():
-        for ret in FileUtilities.wipe_path(path, idle=True):
-            yield ret
+        yield from FileUtilities.wipe_path(path, idle=True)
         yield 0
 
     from bleachbit import Action
