@@ -1,8 +1,8 @@
-#!/usr/bin/env python3
+#!/usr/bin/python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,14 +24,14 @@ GTK graphical user interface
 
 from bleachbit import GuiBasic
 from bleachbit import Cleaner, FileUtilities
-from bleachbit import _, APP_NAME, appicon_path, portable_mode
+from bleachbit import _, APP_NAME, appicon_path, portable_mode, windows10_theme_path
 from bleachbit.Options import options
 from bleachbit.GuiPreferences import PreferencesDialog
 from bleachbit.Cleaner import backends, register_cleaners
 import bleachbit
 from gi.repository import Gtk, Gdk, GObject, GLib, Gio
 
-
+import glob
 import logging
 import os
 import sys
@@ -116,17 +116,26 @@ class Bleachbit(Gtk.Application):
     _auto_exit = False
 
     def __init__(self, uac=True, shred_paths=None, auto_exit=False):
-        if uac and os.name == 'nt' and Windows.elevate_privileges():
-            # privileges escalated in other process
-            sys.exit(0)
+
+        application_id_suffix = self._init_windows_misc(auto_exit, shred_paths, uac)
+        application_id = '{}{}'.format('org.gnome.Bleachbit', application_id_suffix)
         Gtk.Application.__init__(
-            self, application_id='org.gnome.Bleachbit', flags=Gio.ApplicationFlags.FLAGS_NONE)
+            self, application_id=application_id, flags=Gio.ApplicationFlags.FLAGS_NONE)
         GObject.threads_init()
 
+        if auto_exit:
+            # This is used for automated testing of whether the GUI can start.
+            # It is called from assert_execute_console() in windows/setup_py2exe.py
+            self._auto_exit = True        
+
         if shred_paths:
             self._shred_paths = shred_paths
-            return
+
         if os.name == 'nt':
+            # clean up nonce files https://github.com/bleachbit/bleachbit/issues/858
+            import atexit
+            atexit.register(Windows.cleanup_nonce)
+
             # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
             # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
             # https://www.bleachbit.org/forum/074-fails-errors
@@ -135,9 +144,22 @@ class Bleachbit(Gtk.Application):
             except ImportError:
                 logger.exception(
                     _("Error loading the SQLite module: the antivirus software may be blocking it."))
-        if auto_exit:
-            # This is used for automated testing of whether the GUI can start.
-            self._auto_exit = True
+
+    def _init_windows_misc(self, auto_exit, shred_paths, uac):
+        application_id_suffix = ''
+        is_context_menu_executed = auto_exit and shred_paths
+        if os.name == 'nt':
+            if Windows.elevate_privileges(uac):
+                # privileges escalated in other process
+                sys.exit(0)
+
+            if is_context_menu_executed:
+                # When we have a running application and executing the Windows
+                # context menu command we start a new process with new application_id.
+                # That is because the command line arguments of the context menu command
+                # are not passed to the already running instance.
+                application_id_suffix = 'ContextMenuShred'
+        return application_id_suffix
 
     def build_app_menu(self):
         """Build the application menu
@@ -161,7 +183,7 @@ class Bleachbit(Gtk.Application):
                    'makeChaff': self.cb_make_chaff,
                    'shredQuit': self.cb_shred_quit,
                    'preferences': self.cb_preferences_dialog,
-                   'diagnostics': self.diagnostic_dialog,
+                   'systemInformation': self.system_information_dialog,
                    'help': self.cb_help,
                    'about': self.about}
 
@@ -230,6 +252,10 @@ class Bleachbit(Gtk.Application):
             # in portable mode on Windows, the options directory includes
             # executables
             paths.append(bleachbit.options_file)
+            if os.path.isdir(bleachbit.personal_cleaners_dir):
+                paths.append(bleachbit.personal_cleaners_dir)
+            for f in glob.glob(os.path.join(bleachbit.options_dir, "*.bz2")):
+                paths.append(f)
         else:
             paths.append(bleachbit.options_dir)
 
@@ -239,14 +265,14 @@ class Bleachbit(Gtk.Application):
             # aborted
             return
 
-        # in portable mode, rebuild a minimal bleachbit.ini
-        bleachbit.Options.init_configuration()
-
         # Quit the application through the idle loop to allow the worker
         # to delete the files.  Use the lowest priority because the worker
-        # uses the standard priority.  Otherwise, this will quit before
+        # uses the standard priority. Otherwise, this will quit before
         # the files are deleted.
-        GLib.idle_add(self.quit, priority=GObject.PRIORITY_LOW)
+        #
+        # Rebuild a minimal bleachbit.ini when quitting
+        GLib.idle_add(self.quit, None, None, True,
+                      priority=GObject.PRIORITY_LOW)
 
     def cb_wipe_free_space(self, action, param):
         """callback to wipe free space in arbitrary folder"""
@@ -276,7 +302,7 @@ class Bleachbit(Gtk.Application):
 
     def get_about_dialog(self):
         dialog = Gtk.AboutDialog(comments='Program to clean unnecessary files',
-                                 copyright='Copyright (C) 2008-2020 Andrew Ziem',
+                                 copyright='Copyright (C) 2008-2021 Andrew Ziem',
                                  program_name=APP_NAME,
                                  version=bleachbit.APP_VERSION,
                                  website=bleachbit.APP_URL,
@@ -309,16 +335,18 @@ class Bleachbit(Gtk.Application):
         Gtk.Application.do_startup(self)
         self.build_app_menu()
 
-    def quit(self, _action=None, _param=None):
+    def quit(self, _action=None, _param=None, init_configuration=False):
+        if init_configuration:
+            bleachbit.Options.init_configuration()
         self._window.destroy()
 
-    def get_diagnostics_dialog(self):
-        """Show diagnostic information"""
+    def get_system_information_dialog(self):
+        """Show system information dialog"""
         dialog = Gtk.Dialog(_("System information"), self._window)
         dialog.set_default_size(600, 400)
         txtbuffer = Gtk.TextBuffer()
-        from bleachbit import Diagnostic
-        txt = Diagnostic.diagnostic_info()
+        from bleachbit import SystemInformation
+        txt = SystemInformation.get_system_information()
         txtbuffer.set_text(txt)
         textview = Gtk.TextView.new_with_buffer(txtbuffer)
         textview.set_editable(False)
@@ -328,18 +356,17 @@ class Bleachbit(Gtk.Application):
         dialog.vbox.pack_start(swindow, True, True, 0)
         dialog.add_buttons(Gtk.STOCK_COPY, 100,
                            Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
-        return dialog
+        return (dialog, txt)
 
-    def diagnostic_dialog(self, _action, _param):
-        dialog = self.get_diagnostics_dialog()
+    def system_information_dialog(self, _action, _param):
+        dialog, txt = self.get_system_information_dialog()
         dialog.show_all()
         while True:
             rc = dialog.run()
-            if rc == 100:
-                clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
-                clipboard.set_text(txt, -1)
-            else:
+            if rc != 100:
                 break
+            clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+            clipboard.set_text(txt, -1)
         dialog.destroy()
 
     def do_activate(self):
@@ -347,13 +374,15 @@ class Bleachbit(Gtk.Application):
             self._window = GUI(
                 application=self, title=APP_NAME, auto_exit=self._auto_exit)
         self._window.present()
-        if self._auto_exit:
+        if self._shred_paths:
+            GLib.idle_add(GUI.shred_paths, self._window, self._shred_paths, priority=GObject.PRIORITY_LOW)
+            # When we shred paths and auto exit with the Windows Explorer context menu command we close the
+            # application in GUI.shred_paths, because if it is closed from here there are problems.
+            # Most probably this is something related with how GTK handles idle quit calls.
+        elif self._auto_exit:
             GLib.idle_add(self.quit,
                           priority=GObject.PRIORITY_LOW)
             print('Success')
-        if self._shred_paths:
-            GLib.idle_add(GUI.shred_paths, self._window, self._shred_paths, False, True,
-                          priority=GObject.PRIORITY_LOW)
 
 
 class TreeInfoModel:
@@ -457,7 +486,7 @@ class TreeDisplayModel:
         return self.view
 
     def set_cleaner(self, path, model, parent_window, value):
-        """Activate or deactive option of cleaner."""
+        """Activate or deactivate option of cleaner."""
         assert isinstance(value, bool)
         assert isinstance(model, Gtk.TreeStore)
         cleaner_id = None
@@ -520,11 +549,15 @@ class TreeDisplayModel:
 class GUI(Gtk.ApplicationWindow):
     """The main application GUI"""
     _style_provider = None
+    _style_provider_regular = None
+    _style_provider_dark = None
 
     def __init__(self, auto_exit, *args, **kwargs):
         super(GUI, self).__init__(*args, **kwargs)
 
-        self.auto_exit = auto_exit
+        self._show_splash_screen()
+
+        self._auto_exit = auto_exit
 
         self.set_wmclass(APP_NAME, APP_NAME)
         self.populate_window()
@@ -552,53 +585,38 @@ class GUI(Gtk.ApplicationWindow):
                 _('Resetting the configuration file because it is corrupt: %s') % bleachbit.options_file)
             bleachbit.Options.init_configuration()
 
-        if options.get("first_start") and not auto_exit:
-            if os.name == 'posix':
-                self.append_text(
-                    _('Access the application menu by clicking the hamburger icon on the title bar.'))
-                pref = self.get_preferences_dialog()
-                pref.run()
-            if os.name == 'nt':
-                self.append_text(
-                    _('Access the application menu by clicking the logo on the title bar.'))
-            options.set('first_start', False)
-        if os.name == 'nt':
-            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
-            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # http://bleachbit.sourceforge.net/forum/074-fails-errors
-            try:
-                import sqlite3
-            except ImportError as e:
-                self.append_text(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
-
-        if os.name == 'posix' and os.path.expanduser('~') == '/root':
-            self.append_text(
-                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.'))
-        if os.name == 'nt' and options.get('shred'):
-            from win32com.shell.shell import IsUserAnAdmin
-            if not IsUserAnAdmin():
-                self.append_text(
-                    _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
-                self.append_text('\n')
-
         GLib.idle_add(self.cb_refresh_operations)
 
+    def _show_splash_screen(self):
+        if os.name != 'nt':
+            return
+
+        font_conf_file = Windows.get_font_conf_file()
+        if not os.path.exists(font_conf_file):
+            logger.error('No fonts.conf file')
+            return
+
+        has_cache = Windows.has_fontconfig_cache(font_conf_file)
+        if not has_cache:
+            Windows.splash_thread.start()
+
+    def _confirm_delete(self, mention_preview, shred_settings=False):
+        if options.get("delete_confirmation"):
+            return GuiBasic.delete_confirmation_dialog(self, mention_preview, shred_settings=shred_settings)
+        return True
+
     def get_preferences_dialog(self):
         return PreferencesDialog(
             self,
             self.cb_refresh_operations,
             self.set_windows10_theme)
 
-    def shred_paths(self, paths, shred_settings=False, quit_when_done=False):
+    def shred_paths(self, paths, shred_settings=False):
         """Shred file or folders
 
         When shredding_settings=True:
         If user confirms to delete, then returns True.  If user aborts, returns
         False.
-
-        When quit_when_done=True:
-        Always returns False to remove function from the idle queue.
         """
         # create a temporary cleaner object
         backends['_gui'] = Cleaner.create_simple_cleaner(paths)
@@ -607,15 +625,15 @@ class GUI(Gtk.ApplicationWindow):
         operations = {'_gui': ['files']}
         self.preview_or_run_operations(False, operations)
 
-        if GuiBasic.delete_confirmation_dialog(self, mention_preview=False, shred_settings=shred_settings):
+        if self._confirm_delete(False, shred_settings):
             # delete
             self.preview_or_run_operations(True, operations)
             if shred_settings:
                 return True
 
-        if quit_when_done:
+        if self._auto_exit:
             GLib.idle_add(self.close,
-                          priority=GObject.PRIORITY_LOW)
+                              priority=GObject.PRIORITY_LOW)
 
         # user aborted
         return False
@@ -711,10 +729,8 @@ class GUI(Gtk.ApplicationWindow):
         # Disable delete confirmation message.
         # if the option is selected under preference.
 
-        if options.get("delete_confirmation"):
-            if not GuiBasic.delete_confirmation_dialog(self, True):
-                return
-        self.preview_or_run_operations(True)
+        if self._confirm_delete(True):
+            self.preview_or_run_operations(True)
 
     def preview_or_run_operations(self, really_delete, operations=None):
         """Preview operations or run operations (delete files)"""
@@ -723,9 +739,10 @@ class GUI(Gtk.ApplicationWindow):
         from bleachbit import Worker
         self.start_time = None
         if not operations:
-            operations = {}
-            for operation in self.get_selected_operations():
-                operations[operation] = self.get_operation_options(operation)
+            operations = {
+                operation: self.get_operation_options(operation)
+                for operation in self.get_selected_operations()
+            }
         assert isinstance(operations, dict)
         if not operations:  # empty
             GuiBasic.message_dialog(self,
@@ -742,7 +759,7 @@ class GUI(Gtk.ApplicationWindow):
         else:
             self.start_time = time.time()
             worker = self.worker.run()
-            GLib.idle_add(worker.__next__, priority=GLib.PRIORITY_LOW)
+            GLib.idle_add(worker.__next__)
 
     def worker_done(self, worker, really_delete):
         """Callback for when Worker is done"""
@@ -771,19 +788,22 @@ class GUI(Gtk.ApplicationWindow):
         scrolled_window = Gtk.ScrolledWindow()
         scrolled_window.set_policy(
             Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        scrolled_window.set_overlay_scrolling(False)
         self.tree_store = TreeInfoModel()
         display = TreeDisplayModel()
         mdl = self.tree_store.get_model()
         self.view = display.make_view(
             mdl, self, self.context_menu_event)
         self.view.get_selection().connect("changed", self.on_selection_changed)
+        scrollbar_width = scrolled_window.get_vscrollbar().get_preferred_width()[1]
+        self.view.set_margin_right(scrollbar_width) # avoid conflict with scrollbar
         scrolled_window.add(self.view)
         return scrolled_window
 
     def cb_refresh_operations(self):
         """Callback to refresh the list of cleaners"""
         # Is this the first time in this session?
-        if not hasattr(self, 'recognized_cleanerml') and not self.auto_exit:
+        if not hasattr(self, 'recognized_cleanerml') and not self._auto_exit:
             from bleachbit import RecognizeCleanerML
             RecognizeCleanerML.RecognizeCleanerML()
             self.recognized_cleanerml = True
@@ -804,11 +824,47 @@ class GUI(Gtk.ApplicationWindow):
         self.view.expand_all()
 
         # Check for online updates.
-        if bleachbit.online_update_notification_enabled and options.get("check_online_updates") and \
+        if not self._auto_exit and \
+            bleachbit.online_update_notification_enabled and \
+            options.get("check_online_updates") and \
                 not hasattr(self, 'checked_for_updates'):
             self.checked_for_updates = True
             self.check_online_updates()
 
+        # Show information for first start.
+        # (The first start flag is set also for each new version.)
+        if options.get("first_start") and not self._auto_exit:
+            if os.name == 'posix':
+                self.append_text(
+                    _('Access the application menu by clicking the hamburger icon on the title bar.'))
+                pref = self.get_preferences_dialog()
+                pref.run()
+            elif os.name == 'nt':
+                self.append_text(
+                    _('Access the application menu by clicking the logo on the title bar.'))
+            options.set('first_start', False)
+
+        if os.name == 'nt':
+            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
+            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
+            # http://bleachbit.sourceforge.net/forum/074-fails-errors
+            try:
+                import sqlite3
+            except ImportError as e:
+                self.append_text(
+                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
+
+        # Show notice about admin privileges.
+        if os.name == 'posix' and os.path.expanduser('~') == '/root':
+            self.append_text(
+                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.')+'\n')
+        if os.name == 'nt' and options.get('shred'):
+            from win32com.shell.shell import IsUserAnAdmin
+            if not IsUserAnAdmin():
+                self.append_text(
+                    _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
+                self.append_text('\n')
+
         # remove from idle loop (see GObject.idle_add)
         return False
 
@@ -822,7 +878,7 @@ class GUI(Gtk.ApplicationWindow):
             return
 
         # delete
-        if GuiBasic.delete_confirmation_dialog(self, mention_preview=False):
+        if self._confirm_delete(False):
             self.preview_or_run_operations(True, operations)
             return
 
@@ -843,7 +899,7 @@ class GUI(Gtk.ApplicationWindow):
         # context menu applies only to children, not parents
         if len(path) != 2:
             return False
-        # find the seleted option
+        # find the selected option
         model = treeview.get_model()
         option_id = model[path][2]
         cleaner_id = model[path[0]][2]
@@ -999,21 +1055,26 @@ class GUI(Gtk.ApplicationWindow):
         return hbar
 
     def on_configure_event(self, widget, event):
-        # save window position and size
-        screen = self.get_screen()
-        (screen_w, screen_h) = (screen.get_width(), screen.get_height())
         (x, y) = self.get_position()
-        if x >= screen_w or y >= screen_h:
-            monitor = screen.get_monitor_at_window(self.get_window())
-            g = screen.get_monitor_geometry(monitor)
-            print("Fixup moving the window off the screen: window (x, y) = {}, screen (w, h) = {}, monitor (x, y) = {}, (w, h) = {}".format(
-                (x, y), (screen_w, screen_h),
-                (g.x, g.y), (g.width, g.height)))
-            self.move(g.x, g.y)
-            return True
+        (width, height) = self.get_size()
+
+        # fixup maximized window position:
+        # on Windows if a window is maximized on a secondary monitor it is moved off the screen
+        if 'nt' == os.name:
+            window = self.get_window()
+            if window.get_state() & Gdk.WindowState.MAXIMIZED != 0:
+                screen = self.get_screen()
+                monitor_num = screen.get_monitor_at_window(window)
+                g = screen.get_monitor_geometry(monitor_num)
+                if x < g.x or x >= g.x + g.width or y < g.y or y >= g.y + g.height:
+                    logger.debug("Maximized window {}+{}: monitor ({}) geometry = {}+{}".format(
+                        (x, y), (width, height), monitor_num, (g.x, g.y), (g.width, g.height)))
+                    self.move(g.x, g.y)
+                    return True
+
+        # save window position and size
         options.set("window_x", x, commit=False)
         options.set("window_y", y, commit=False)
-        (width, height) = self.get_size()
         options.set("window_width", width, commit=False)
         options.set("window_height", height, commit=False)
         return False
@@ -1032,12 +1093,32 @@ class GUI(Gtk.ApplicationWindow):
         return False
 
     def on_show(self, widget):
+
+        if 'nt' == os.name and Windows.splash_thread.is_alive():
+            Windows.splash_thread.join(0)
+
         # restore window position, size and state
-        if options.has_option("window_x") and options.has_option("window_y"):
-            self.move(options.get("window_x"), options.get("window_y"))
-        if options.has_option("window_width") and options.has_option("window_height"):
-            self.resize(options.get("window_width"),
-                        options.get("window_height"))
+        if not options.get('remember_geometry'):
+            return
+        if options.has_option("window_x") and options.has_option("window_y") and \
+           options.has_option("window_width") and options.has_option("window_height"):
+            r = Gdk.Rectangle()
+            (r.x, r.y) = (options.get("window_x"), options.get("window_y"))
+            (r.width, r.height) = (options.get(
+                "window_width"), options.get("window_height"))
+
+            screen = self.get_screen()
+            monitor_num = screen.get_monitor_at_point(r.x, r.y)
+            g = screen.get_monitor_geometry(monitor_num)
+
+            # only restore position and size if window left corner
+            # is within the closest monitor
+            if r.x >= g.x and r.x < g.x + g.width and \
+               r.y >= g.y and r.y < g.y + g.height:
+                logger.debug("closest monitor ({}) geometry = {}+{}, window geometry = {}+{}".format(
+                    monitor_num, (g.x, g.y), (g.width, g.height), (r.x, r.y), (r.width, r.height)))
+                self.move(r.x, r.y)
+                self.resize(r.width, r.height)
         if options.get("window_fullscreen"):
             self.fullscreen()
         elif options.get("window_maximized"):
@@ -1047,33 +1128,29 @@ class GUI(Gtk.ApplicationWindow):
         """Toggle the Windows 10 theme"""
         if not 'nt' == os.name:
             return
-        exec_path = os.path.dirname(sys.executable)
-        windows_10_theme_exe_path = os.path.normpath(
-            os.path.join(exec_path, 'themes/windows10/gtk.css'))
-        windows_10_theme_source_path = "themes/windows10/gtk.css"
-        load_path = None
 
-        if os.path.exists(windows_10_theme_exe_path):
-            load_path = windows_10_theme_exe_path
-        elif os.path.exists(windows_10_theme_source_path):
-            load_path = windows_10_theme_source_path
-
-        if not self._style_provider:
-            self._style_provider = Gtk.CssProvider()
-
-            if not load_path:
-                logger.warning('cannot find windows10/gtk.css')
-                return
-
-            self._style_provider.load_from_path(load_path)
+        if not self._style_provider_regular:
+            self._style_provider_regular = Gtk.CssProvider()
+            self._style_provider_regular.load_from_path(
+                os.path.join(windows10_theme_path, 'gtk.css'))
+        if not self._style_provider_dark:
+            self._style_provider_dark = Gtk.CssProvider()
+            self._style_provider_dark.load_from_path(
+                os.path.join(windows10_theme_path, 'gtk-dark.css'))
 
         screen = Gdk.Display.get_default_screen(Gdk.Display.get_default())
-        if options.get("win10_theme"):
-            Gtk.StyleContext.add_provider_for_screen(
-                screen, self._style_provider, 600)
-        else:
+        if self._style_provider is not None:
             Gtk.StyleContext.remove_provider_for_screen(
                 screen, self._style_provider)
+        if options.get("win10_theme"):
+            if options.get("dark_mode"):
+                self._style_provider = self._style_provider_dark
+            else:
+                self._style_provider = self._style_provider_regular
+            Gtk.StyleContext.add_provider_for_screen(
+                screen, self._style_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+        else:
+            self._style_provider = None
 
     def populate_window(self):
         """Create the main application window"""
