@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,8 +25,14 @@ Run all test suites
 import os
 import unittest
 import sys
+import tempfile
+import shutil
+
 
 if __name__ == '__main__':
+    testdir = tempfile.mkdtemp(prefix='TestAll '+__name__)
+    os.environ['BLEACHBIT_TEST_OPTIONS_DIR'] = testdir
+    
     print("""You should use the unittest discovery, it's much nicer:
     python -m unittest discover -p Test*.py                       # run all tests
     python -m unittest tests.TestCLI                              # run only the CLI tests
@@ -34,4 +40,9 @@ if __name__ == '__main__':
     suite = unittest.defaultTestLoader.discover(
         os.getcwd(), pattern='Test*.py')
     success = unittest.TextTestRunner(verbosity=2).run(suite).wasSuccessful()
+    
+    del os.environ['BLEACHBIT_TEST_OPTIONS_DIR']
+    if os.path.exists(testdir):
+        shutil.rmtree(testdir)
+    
     sys.exit(success == False)
