@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -103,6 +103,9 @@ class OptionsTestCase(common.BleachbitTestCase):
 
     def test_init_configuration(self):
         """Test for init_configuration()"""
+        if os.path.exists(bleachbit.options_file):
+            os.remove(bleachbit.options_file)
+        self.assertNotExists(bleachbit.options_file)
         bleachbit.Options.init_configuration()
         self.assertExists(bleachbit.options_file)
 
