@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2021 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -199,7 +199,7 @@ INSERT INTO moz_icons VALUES(3,'fake-favicon-uri:https://support.mozilla.org/en-
 CREATE TABLE moz_icons_to_pages ( page_id INTEGER NOT NULL, icon_id INTEGER NOT NULL, expire_ms INTEGER NOT NULL DEFAULT 0, PRIMARY KEY (page_id, icon_id), FOREIGN KEY (page_id) REFERENCES moz_pages_w_icons ON DELETE CASCADE, FOREIGN KEY (icon_id) REFERENCES moz_icons ON DELETE CASCADE ) WITHOUT ROWID;
 INSERT INTO moz_icons_to_pages VALUES(1,1,1603222567764);
 INSERT INTO moz_icons_to_pages VALUES(2,2,1603222567764);
-INSERT INTO moz_icons_to_pages VALUES(3,3,1603222567764); 
+INSERT INTO moz_icons_to_pages VALUES(3,3,1603222567764);
 CREATE TABLE moz_pages_w_icons ( id INTEGER PRIMARY KEY, page_url TEXT NOT NULL, page_url_hash INTEGER NOT NULL );
 INSERT INTO moz_pages_w_icons VALUES(1,'https://support.mozilla.org/en-US/products/firefox',47357795150914);
 INSERT INTO moz_pages_w_icons VALUES(2,'https://support.mozilla.org/en-US/products/firefox2',473577951509142);
@@ -268,7 +268,6 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
         FileUtilities.execute_sqlite3(
             os.path.join(dir_firefox_default, 'favicons.sqlite'), firefox78_favicons_sql)
 
-
     def tearDown(self):
         """Remove test browser files."""
         shutil.rmtree(self.dir_base)
@@ -307,8 +306,8 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
         clean_func(filename)
         options.set('shred', True, commit=False)
         self.assertTrue(options.get('shred'))
-        options.set('shred', old_shred, commit=False)
         clean_func(filename)
+        options.set('shred', old_shred, commit=False)
         self.assertExists(filename)
 
         # check
@@ -342,9 +341,7 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
             conn = sqlite3.connect(filename)
             c = conn.cursor()
             c.execute('select id from urls')
-            ids = []
-            for row in c:
-                ids.append(row[0])
+            ids = [row[0] for row in c]
             # id 1 is sqlite.org which is not a bookmark and should be cleaned
             # id 2 is www.bleachbit.org which is a bookmark and should
             # be cleaned
@@ -364,9 +361,7 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
             conn = sqlite3.connect(filename)
             c = conn.cursor()
             c.execute('select id from keywords')
-            ids = []
-            for row in c:
-                ids.append(row[0])
+            ids = [row[0] for row in c]
             self.assertEqual(ids, [2, 3, 4, 5, 6])
 
         self.sqlite_clean_helper(None, "google-chrome/Default/Web Data", Special.delete_chrome_keywords,
@@ -386,7 +381,7 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
             sql_palemoon, None, Special.delete_mozilla_url_history)
 
     def test_delete_mozilla_favicons(self):
-        """Test for delete_mozilla_url_history"""
+        """Test for delete_mozilla_favicons"""
         self.sqlite_clean_helper(
             None, "firefox/default-release/favicons.sqlite", Special.delete_mozilla_favicons)
 
@@ -409,21 +404,15 @@ class SpecialTestCase(common.BleachbitTestCase, SpecialAssertions):
 
         os.unlink(path)
 
-    def test_get_sqlite(self):
-        """Unit test for get_sqlite()"""
+    def test_get_sqlite_int(self):
+        """Unit test for get_sqlite_int()"""
         sql = """CREATE TABLE meta(key LONGVARCHAR NOT NULL UNIQUE PRIMARY KEY,value LONGVARCHAR);
-INSERT INTO "meta" VALUES('version','20');
-INSERT INTO "meta" VALUES('version_str','aaa');
-"""
+INSERT INTO "meta" VALUES('version','20');"""
         # create test file
         filename = self.mkstemp(prefix='bleachbit-test-sqlite')
         FileUtilities.execute_sqlite3(filename, sql)
         self.assertExists(filename)
         # run the test
-        ver = Special.get_sqlite(
+        ver = Special.get_sqlite_int(
             filename, 'select value from meta where key="version"')
         self.assertEqual(ver, [20])
-
-        ver = Special.get_sqlite(
-            filename, 'select value from meta where key="version_str"', None, str)
-        self.assertEqual(ver, ["aaa"])
