@@ -1,6 +1,6 @@
 """
 BleachBit
-Copyright (C) 2008-2019 Andrew Ziem
+Copyright (C) 2008-2021 Andrew Ziem
 https://www.bleachbit.org
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,7 +24,14 @@ import os
 import shutil
 import subprocess
 import sys
+import time
+import win_unicode_console
+import xml.dom.minidom
 
+from windows.NsisUtilities import write_nsis_expressions_to_files
+
+setup_encoding = sys.stdout.encoding
+win_unicode_console.enable()
 logger = logging.getLogger('setup_py2exe')
 logger.setLevel(logging.INFO)
 ch = logging.StreamHandler()
@@ -45,9 +52,10 @@ logger.info('ROOT_DIR ' + ROOT_DIR)
 sys.path.append(ROOT_DIR)
 
 BB_VER = None
-GTK_DIR = 'C:\\Python27\\Lib\\site-packages\\gnome\\'
+GTK_DIR = sys.exec_prefix + '\\Lib\\site-packages\\gnome\\'
 NSIS_EXE = 'C:\\Program Files (x86)\\NSIS\\makensis.exe'
 NSIS_ALT_EXE = 'C:\\Program Files\\NSIS\\makensis.exe'
+SHRED_REGEX_KEY = 'AllFilesystemObjects\\shell\\shred.bleachbit'
 if not os.path.exists(NSIS_EXE) and os.path.exists(NSIS_ALT_EXE):
     logger.info('NSIS found in alternate location: ' + NSIS_ALT_EXE)
     NSIS_EXE = NSIS_ALT_EXE
@@ -57,11 +65,12 @@ SZ_EXE = 'C:\\Program Files\\7-Zip\\7z.exe'
 # mpass=passes for deflate encoder
 # mfb=number of fast bytes
 # bso0 bsp0 quiet output
+# 7-Zip Command Line Reverence Wizard: https://axelstudios.github.io/7z/#!/
 SZ_OPTS = '-tzip -mm=Deflate -mfb=258 -mpass=7 -bso0 -bsp0'  # best compression
 if fast:
     # fast compression
     SZ_OPTS = '-tzip -mx=1 -bso0 -bsp0'
-UPX_EXE = ROOT_DIR + '\\upx394w\\upx.exe'
+UPX_EXE = ROOT_DIR + '\\upx\\upx.exe'
 UPX_OPTS = '--best --crp-ms=999999 --nrv2e'
 
 
@@ -96,7 +105,6 @@ def check_exist(path, msg=None):
         logger.warning(path + ' not found')
         if msg:
             logger.warning(msg)
-        import time
         time.sleep(5)
 
 
@@ -111,7 +119,7 @@ def assert_module(module):
 
 def assert_execute(args, expected_output):
     """Run a command and check it returns the expected output"""
-    actual_output = subprocess.check_output(args)
+    actual_output = subprocess.check_output(args).decode(setup_encoding)
     if -1 == actual_output.find(expected_output):
         raise RuntimeError('When running command {} expected output {} but got {}'.format(
             args, expected_output, actual_output))
@@ -129,9 +137,9 @@ def run_cmd(cmd):
     p = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     stdout, stderr = p.communicate()
-    logger.info(stdout)
+    logger.info(stdout.decode(setup_encoding))
     if stderr:
-        logger.error(stderr)
+        logger.error(stderr.decode(setup_encoding))
 
 
 def sign_code(filename):
@@ -146,7 +154,7 @@ def sign_code(filename):
 def get_dir_size(start_path='.'):
     # http://stackoverflow.com/questions/1392413/calculating-a-directory-size-using-python
     total_size = 0
-    for dirpath, dirnames, filenames in os.walk(start_path):
+    for dirpath, _dirnames, filenames in os.walk(start_path):
         for f in filenames:
             fp = os.path.join(dirpath, f)
             total_size += os.path.getsize(fp)
@@ -213,29 +221,57 @@ def build():
     if not os.path.exists('dist'):
         os.makedirs('dist')
 
+    logger.info('Copying GTK helpers')
+    for exe in ('gspawn-win32-helper.exe', 'gspawn-win32-helper-console.exe', 'fc-cache.exe'):
+        shutil.copyfile(os.path.join(GTK_DIR, exe), os.path.join('dist', exe))
+
     logger.info('Copying GTK files and icon')
     copytree(GTK_DIR + '\\etc', 'dist\\etc')
     copytree(GTK_DIR + '\\lib', 'dist\\lib')
-    for subpath in ['fontconfig', 'fonts', 'icons', 'themes']:
-        copytree(os.path.join(GTK_DIR, 'share', subpath), 'dist\\share\\' + subpath)
+
+    logger.info('Remove windows fonts dir from fonts.conf file')
+
+    # fonts are not needed https://github.com/bleachbit/bleachbit/issues/863
+    for subpath in ['fontconfig', 'icons', 'themes']:
+        copytree(os.path.join(GTK_DIR, 'share', subpath),
+                 'dist\\share\\' + subpath)
     SCHEMAS_DIR = 'share\\glib-2.0\\schemas'
     os.makedirs(os.path.join('dist', SCHEMAS_DIR))
-    shutil.copyfile(os.path.join(GTK_DIR, SCHEMAS_DIR, 'gschemas.compiled'), os.path.join('dist', SCHEMAS_DIR, 'gschemas.compiled'))
+    shutil.copyfile(os.path.join(GTK_DIR, SCHEMAS_DIR, 'gschemas.compiled'),
+                    os.path.join('dist', SCHEMAS_DIR, 'gschemas.compiled'))
     shutil.copyfile('bleachbit.png',  'dist\\share\\bleachbit.png')
+    # for pop-up notification
+    shutil.copyfile('windows\\bleachbit.ico',  'dist\\share\\bleachbit.ico')
     for dll in glob.glob1(GTK_DIR, '*.dll'):
-        shutil.copyfile(os.path.join(GTK_DIR,dll), 'dist\\'+dll)
+        shutil.copyfile(os.path.join(GTK_DIR, dll), 'dist\\'+dll)
 
     os.mkdir('dist\\data')
     shutil.copyfile('data\\app-menu.ui', 'dist\\data\\app-menu.ui')
 
+    logger.info('Copying themes')
+    copytree('themes', 'dist\\themes')
+
     logger.info('Copying CA bundle')
     import requests
-    shutil.copyfile(requests.utils.DEFAULT_CA_BUNDLE_PATH, os.path.join('dist', 'cacert.pem'))
+    shutil.copyfile(requests.utils.DEFAULT_CA_BUNDLE_PATH,
+                    os.path.join('dist', 'cacert.pem'))
+
+    dist_locale_dir = r'dist\share\locale'
+    logger.info('Copying GTK localizations')
+    shutil.rmtree(dist_locale_dir, ignore_errors=True)
+    os.makedirs(dist_locale_dir)
+    locale_dir = os.path.join(GTK_DIR, 'share\\locale\\')
+    for f in recursive_glob(locale_dir, ['gtk30.mo']):
+        if not f.startswith(locale_dir):
+            continue
+        rel_f = f[len(locale_dir):]
+        os.makedirs(os.path.join(dist_locale_dir, os.path.dirname(rel_f)))
+        shutil.copyfile(f, os.path.join(dist_locale_dir, rel_f))
+    assert_exist(os.path.join(dist_locale_dir, r'es\LC_MESSAGES\gtk30.mo'))
 
     logger.info('Copying BleachBit localizations')
-    shutil.rmtree('dist\\share\\locale', ignore_errors=True)
-    copytree('locale', 'dist\\share\\locale')
-    assert_exist('dist\\share\\locale\\es\\LC_MESSAGES\\bleachbit.mo')
+    copytree('locale', dist_locale_dir)
+    assert_exist(os.path.join(dist_locale_dir, r'es\LC_MESSAGES\bleachbit.mo'))
 
     logger.info('Copying BleachBit cleaners')
     if not os.path.exists('dist\\share\\cleaners'):
@@ -250,6 +286,9 @@ def build():
     logger.info('Copying license')
     shutil.copy('COPYING', 'dist')
 
+    logger.info('Copying msvcr100.dll')
+    shutil.copy('C:\\WINDOWS\\system32\\msvcr100.dll', 'dist\\msvcr100.dll')
+
     sign_code('dist\\bleachbit.exe')
     sign_code('dist\\bleachbit_console.exe')
 
@@ -260,7 +299,7 @@ def build():
 def delete_unnecessary():
     logger.info('Deleting unnecessary files')
     # Remove SVG to reduce space and avoid this error
-    # Error loading theme icon 'dialog-warning' for stock: Unable to load image-loading module: C:/Python27/Lib/site-packages/gtk-2.0/runtime/lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-svg.dll: `C:/Python27/Lib/site-packages/gtk-2.0/runtime/lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-svg.dll': The specified module could not be found.
+    # Error loading theme icon 'dialog-warning' for stock: Unable to load image-loading module: C:/PythonXY/Lib/site-packages/gtk-2.0/runtime/lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-svg.dll: `C:/PythonXY/Lib/site-packages/gtk-2.0/runtime/lib/gdk-pixbuf-2.0/2.10.0/loaders/libpixbufloader-svg.dll': The specified module could not be found.
     # https://bugs.launchpad.net/bleachbit/+bug/1650907
     delete_paths = [
         r'_win32sysloader.pyd',
@@ -301,14 +340,15 @@ def delete_icons():
         'edit-clear-all.png',
         'edit-delete.png',
         'edit-find.png',
-        'list-add-symbolic.svg', # spin box in chaff dialog
-        'list-remove-symbolic.svg', # spin box in chaff dialog
-        'pan-down-symbolic.svg', # there is no pan-down.png
-        'process-stop.png', # abort on toolbar
-        'window-close-symbolic.svg', # png does not get used
-        'window-maximize-symbolic.svg', # no png
-        'window-minimize-symbolic.svg', # no png
-        'window-restore-symbolic.svg' # no png
+        'list-add-symbolic.svg',  # spin box in chaff dialog
+        'list-remove-symbolic.svg',  # spin box in chaff dialog
+        'pan-down-symbolic.svg',  # there is no pan-down.png
+        'pan-end-symbolic.svg',  # there is no pan-end.png
+        'process-stop.png',  # abort on toolbar
+        'window-close-symbolic.svg',  # png does not get used
+        'window-maximize-symbolic.svg',  # no png
+        'window-minimize-symbolic.svg',  # no png
+        'window-restore-symbolic.svg'  # no png
     ]
     strip_list = recursive_glob(r'dist\share\icons', ['*.png', '*.svg'])
     for f in strip_list:
@@ -317,6 +357,7 @@ def delete_icons():
         else:
             logger.info('keeping whitelisted icon: %s', f)
 
+
 def remove_empty_dirs(root):
     """Remove empty directories"""
     import scandir
@@ -327,6 +368,7 @@ def remove_empty_dirs(root):
                 logger.info('Deleting empty directory: %s' % entry.path)
                 os.rmdir(entry.path)
 
+
 @count_size_improvement
 def clean_translations():
     """Clean translations (localizations)"""
@@ -347,12 +389,41 @@ def clean_translations():
 def strip():
     logger.info('Stripping executables')
     strip_list = recursive_glob('dist', ['*.dll', '*.pyd'])
-    strip_whitelist = ['_sqlite3']
+    strip_whitelist = ['_sqlite3.dll']
     strip_files_str = [f for f in strip_list if os.path.basename(
         f) not in strip_whitelist]
-    cmd = 'strip.exe --strip-debug --discard-all --preserve-dates ' + \
-        ' '.join(strip_files_str)
-    run_cmd(cmd)
+    # Process each file individually in case it is locked. See
+    # https://github.com/bleachbit/bleachbit/issues/690
+    for strip_file in strip_files_str:
+        if os.path.exists('strip.tmp'):
+            os.remove('strip.tmp')
+        if not os.path.exists(strip_file):
+            logger.error('%s does not exist before stripping', strip_file)
+            continue
+        cmd = 'strip.exe --strip-debug --discard-all --preserve-dates -o strip.tmp %s' % strip_file
+        run_cmd(cmd)
+        if not os.path.exists(strip_file):
+            logger.error('%s does not exist after stripping', strip_file)
+            continue
+        if not os.path.exists('strip.tmp'):
+            logger.warning('strip.tmp missing while processing %s', strip_file)
+            continue
+        error_counter = 0
+        while error_counter < 100:
+            try:
+                os.remove(strip_file)
+            except PermissionError:
+                logger.warning(
+                    'permissions error while removing %s', strip_file)
+                time.sleep(.1)
+                error_counter += 1
+            else:
+                break
+        if error_counter > 1:
+            logger.warning('error counter %d while removing %s',
+                           error_counter, strip_file)
+        if not os.path.exists(strip_file):
+            os.rename('strip.tmp', strip_file)
 
 
 @count_size_improvement
@@ -362,11 +433,14 @@ def upx():
         return
 
     if not os.path.exists(UPX_EXE):
-        logger.warning('To compress executables, install UPX to: ' + UPX_EXE)
+        logger.warning(
+            'UPX not found. To compress executables, install UPX to: ' + UPX_EXE)
         return
 
     logger.info('Compressing executables')
-    upx_files = recursive_glob('dist', ['*.exe', '*.dll', '*.pyd'])
+    # Do not compress bleachbit.exe and bleachbit_console.exe to avoid false positives
+    # with antivirus software. Not much is space with gained with these small files, anyway.
+    upx_files = recursive_glob('dist', ['*.dll', '*.pyd'])
     cmd = '{} {} {}'.format(UPX_EXE, UPX_OPTS, ' '.join(upx_files))
     run_cmd(cmd)
 
@@ -405,7 +479,8 @@ def recompress_library():
     os.remove('dist\\library.zip')
 
     # clean unused modules from library.zip
-    delete_paths = ['distutils']
+    delete_paths = ['distutils', 'plyer\\platforms\\android',
+                    'plyer\\platforms\\ios', 'plyer\\platforms\\linux', 'plyer\\platforms\\macosx']
     for p in delete_paths:
         shutil.rmtree(os.path.join('dist', 'library', p))
 
@@ -415,9 +490,9 @@ def recompress_library():
     p = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE, cwd='dist\\library')
     stdout, stderr = p.communicate()
-    logger.info(stdout)
+    logger.info(stdout.decode(setup_encoding))
     if stderr:
-        logger.error(stderr)
+        logger.error(stderr.decode(setup_encoding))
 
     file_size_new = os.path.getsize('dist\\library.zip')
     file_size_diff = file_size_old - file_size_new
@@ -476,7 +551,8 @@ def nsis(opts, exe_name, nsi_path):
         logger.info('Deleting old file: ' + exe_name)
         os.remove(exe_name)
     cmd = NSIS_EXE + \
-        ' {} /DVERSION={} {}'.format(opts, BB_VER, nsi_path)
+        ' {} /DVERSION={} /DSHRED_REGEX_KEY={} {}'.format(
+            opts, BB_VER, SHRED_REGEX_KEY, nsi_path)
     run_cmd(cmd)
     assert_exist(exe_name)
     sign_code(exe_name)
@@ -490,18 +566,21 @@ def package_installer(nsi_path=r'windows\bleachbit.nsi'):
         return
 
     logger.info('Building installer')
+
+    write_nsis_expressions_to_files()
+
     exe_name = 'windows\\BleachBit-{0}-setup.exe'.format(BB_VER)
-    #Was:
+    # Was:
     #opts = '' if fast else '/X"SetCompressor /FINAL zlib"'
-    #Now: Done in NSIS file!
-    opts = '' if fast else '/Dpackhdr /DCompressor'
+    # Now: Done in NSIS file!
+    opts = '' if fast else '/V3 /Dpackhdr /DCompressor'
     nsis(opts, exe_name, nsi_path)
 
     if not fast:
-        #Was:
-        #nsis('/DNoTranslations',
-        #Now: Compression gets now done in NSIS file!
-        nsis('/DNoTranslations /Dpackhdr /DCompressor',
+        # Was:
+        # nsis('/DNoTranslations',
+        # Now: Compression gets now done in NSIS file!
+        nsis(opts + ' /DNoTranslations',
              'windows\\BleachBit-{0}-setup-English.exe'.format(BB_VER),
              nsi_path)
 
@@ -520,6 +599,9 @@ if '__main__' == __name__:
     logger.info('Getting BleachBit version')
     import bleachbit
     BB_VER = bleachbit.APP_VERSION
+    build_number = os.getenv('APPVEYOR_BUILD_NUMBER')
+    if build_number:
+        BB_VER = '%s.%s' % (BB_VER, build_number)
     logger.info('BleachBit version ' + BB_VER)
 
     environment_check()
